<?php

class Hiconix
{
    private $db;
    private $config;
    private $req;
    private $header;
    private $logfile;
    private $data = array();
    private $total_prods = 0;
    private $loaded_prods = 0;
    private $brand = false;
    private $brand_id = 0;
    private $manid = 0;
    private $category = false;
    private $topcat = 0;
    private $subcat = 0;
    private $langid = 0;
    private $product = false;
    private $prod_id = 0;
    private $attributes = false;
    private $attrgrid = false;
    private $attrmap = false;

    private $pairs = array(); // все комплекты
    private $pairs_prod = array(); // все товары, образующие комплекты
    private $addedPairs = array(); // обработанные комплекты

    public function __construct(&$db)
    {
        $this->db = $db;
        $this->db->query("SET SESSION wait_timeout = 3600");
        $this->config = $this->getConfig();
        $this->req = $this->buildReq();
        $this->header = 'Authorization: Token ' . $this->config['token'];
        $this->logfile = DIR_LOGS . 'hiconix.log';
        $langid = $db->query("
			SELECT `language_id` AS langid FROM `" . DB_PREFIX . "language`
			WHERE `code` = 'ru' OR `code` = 'ru-ru'
		");
        $this->langid = (int)$langid->row['langid'];
    }

    private function terminate()
    {
        $this->repairCategories();
        exit(0);
    }

    private function log($code, $pcent = false, $msg = false)
    {
        if (!file_exists($this->logfile)) {
            //$this->terminate();
            return;
        }
        $testlog = json_decode(file_get_contents($this->logfile), true);
        if (!isset($testlog['code']) || $testlog['code'] == 'ERROR') {
            //$this->terminate();
            return;
        }
        $retval = array('code' => $code);
        if ($pcent !== false) $retval['percent'] = $pcent;
        if ($msg) $retval['msg'] = $msg;
        file_put_contents($this->logfile, json_encode($retval));
    }

    private function getConfig()
    {
        $conf = $this->db->query("
			SELECT `value` FROM `" . DB_PREFIX . "setting`
			WHERE `key` = 'hiconix_settings'
		");
        if (isset($conf->row['value'])) {
            return $this->config = json_decode($conf->row['value'], true);
        } else {
            file_put_contents($this->logfile, json_encode(array('code' => 'ERROR', 'msg' => 'Настройки не заданы')));
        }
    }

    private function buildReq()
    {
        $retval = array();
        foreach ($this->config['brands'] as $idx => $brand) {
            if (empty($brand['load'])) {
                continue;
            }
            $retval[$brand['code']] = array(
                'idx' => $idx,
                'name' => $brand['name'],
                'path' => 'https://api.hiconix.ru/api/v1/brands?params[brand]=' . $brand['code'],
                'cat' => (int)$brand['category']
            );
        }
        return $retval;
    }

    private function repairCategories($parent_id = 0)
    {
        $query = $this->db->query("SELECT * FROM " . DB_PREFIX . "category WHERE parent_id = '" . (int)$parent_id . "'");
        foreach ($query->rows as $category) {
            $this->db->query("DELETE FROM `" . DB_PREFIX . "category_path` WHERE category_id = '" . (int)$category['category_id'] . "'");
            $level = 0;
            $query = $this->db->query("SELECT * FROM `" . DB_PREFIX . "category_path` WHERE category_id = '" . (int)$parent_id . "' ORDER BY level ASC");
            foreach ($query->rows as $result) {
                $this->db->query("
					INSERT INTO `" . DB_PREFIX . "category_path`
					SET category_id = '" . (int)$category['category_id'] . "',
						`path_id` = '" . (int)$result['path_id'] . "',
						level = '" . (int)$level . "'
				");
                $level++;
            }
            $this->db->query("
				REPLACE INTO `" . DB_PREFIX . "category_path`
				SET category_id = '" . (int)$category['category_id'] . "',
					`path_id` = '" . (int)$category['category_id'] . "',
					level = '" . (int)$level . "'
			");
            $this->repairCategories($category['category_id']);
        }
    }

    private function getImage($path, $dest = 'product')
    {
        $db_img = 'catalog/hiconix/' . $dest . '/';
        $file = explode('/', $path);
        $file = end($file);
        $file = explode('?', $file);
        $file = $file[0];
        $dir = DIR_IMAGE . $db_img;
        if (!is_dir($dir)) {
            mkdir($dir, 0777, true);
        }
        $db_img .= $file;
        if (file_exists(DIR_IMAGE . $db_img)) {
            return $db_img;
        }
        $curl = curl_init();
        curl_setopt($curl, CURLOPT_URL, $path);
        curl_setopt($curl, CURLOPT_FOLLOWLOCATION, true);
        curl_setopt($curl, CURLOPT_MAXREDIRS, 10);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_HEADER, false);
        $imd = curl_exec($curl);
        if (!curl_errno($curl)) {
            if (curl_getinfo($curl, CURLINFO_HTTP_CODE) == 200) {
                if ($imd) {
                    file_put_contents(DIR_IMAGE . $db_img, $imd);
                    curl_close($curl);
                    return $db_img;
                }
            }
        }
        curl_close($curl);
        return '';
    }

    private function setImages($pid, $images, $upd = false)
    {
        if ($upd) {
            $tmp = array();
            foreach ($images as $i) {
                $tmp[] = $i['url'];
            }
            $images = $tmp;
            $tmp = $this->db->query("
				SELECT `product_image_id` AS pimid, `image_src`
				FROM `" . DB_PREFIX . "product_image`
				WHERE `product_id` = " . $pid . "
			");
            $rmid = array();
            foreach ($tmp->rows as $r) {
                $idx = array_search($r['image_src'], $images);
                if ($idx === false) {
                    $rmid[] = $r['pimid'];
                    continue;
                }
                unset($images[$idx]);
            }
            if ($rmid) {
                $this->db->query("
					DELETE FROM `" . DB_PREFIX . "product_image`
					WHERE `product_image_id` IN(" . implode(',', $rmid) . ")
				");
            }
            foreach ($images as $i) {
                $img = $this->getImage($i);
                $this->db->query("
					INSERT INTO `" . DB_PREFIX . "product_image`
					SET `product_id` = " . $pid . ",
						`image` = '" . $this->db->escape($img) . "',
						`image_src` = '" . $this->db->escape($i) . "'
				");
            }
        } else {
            foreach ($images as $i) {
                $img = $this->getImage($i['url']);
                $this->db->query("
					INSERT INTO `" . DB_PREFIX . "product_image`
					SET `product_id` = " . $pid . ",
						`image` = '" . $this->db->escape($img) . "',
						`image_src` = '" . $this->db->escape($i['url']) . "'
				");
            }
        }

    }

    private function setManufacturer()
    {
        $test = $this->db->query("
			SELECT `manufacturer_id` AS manid
			FROM `" . DB_PREFIX . "manufacturer`
			WHERE `hix_mid` = " . $this->brand['id'] . "
		");
        if (isset($test->row['manid'])) {
            $mid = (int)$test->row['manid'];
            $this->db->query("
				DELETE FROM `" . DB_PREFIX . "manufacturer_to_store`
				WHERE `manufacturer_id` = " . $mid . "
					AND `store_id` = " . (int)$this->config['store'] . "
			");
            $this->db->query("
				INSERT INTO `" . DB_PREFIX . "manufacturer_to_store`
				SET `manufacturer_id` = " . $mid . ",
				`store_id` = " . (int)$this->config['store'] . "
			");
            return $mid;
        }
        $image = $this->getImage($this->brand['logo'], 'brands');
        $this->db->query("
			INSERT INTO `" . DB_PREFIX . "manufacturer`
			SET `hix_mid` = " . $this->brand['id'] . ",
			`name` = '" . $this->db->escape($this->brand['name']) . "',
			`image` = '" . $this->db->escape($image) . "'
		");
        $mid = (int)$this->db->getLastId();
        $this->db->query("
			INSERT INTO `" . DB_PREFIX . "manufacturer_description`
			SET `manufacturer_id` = " . $mid . ",
			`name` = '" . $this->db->escape($this->brand['name']) . "',
			`description` = '" . $this->db->escape($this->brand['description']) . "',
			`language_id` = " . $this->langid . ",
			`meta_title` = '" . $this->db->escape($this->brand['name']) . "'
		");
        $this->db->query("
			INSERT INTO `" . DB_PREFIX . "manufacturer_to_store`
			SET `manufacturer_id` = " . $mid . ",
			`store_id` = " . (int)$this->config['store'] . "
		");
        $this->db->query("
			DELETE FROM `" . DB_PREFIX . "url_alias`
			WHERE `query` = 'manufacturer_id=" . $mid . "'
		");
        $slug = 'brand_' . $this->brand['slug'];
        $test = $this->db->query("
			SELECT `query` FROM `" . DB_PREFIX . "url_alias`
			WHERE `keyword` = '" . $this->db->escape($slug) . "'
		");
        if (isset($test->row['query'])) {
            $slug .= '_m' . $mid;
        }
        $this->db->query("
			INSERT INTO `" . DB_PREFIX . "url_alias`
			SET `query` = 'manufacturer_id=" . $mid . "',
				`keyword` = '" . $this->db->escape($slug) . "'
		");
        return $mid;
    }

    private function getProducts($id_cat)
    {
        $curl = curl_init();
        curl_setopt($curl, CURLOPT_URL, 'https://api.hiconix.ru/api/v2/categories?id[]=' . $id_cat);
        curl_setopt($curl, CURLOPT_HTTPHEADER, array($this->header));
        curl_setopt($curl, CURLOPT_FOLLOWLOCATION, true);
        curl_setopt($curl, CURLOPT_MAXREDIRS, 10);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_HEADER, false);
        $json = curl_exec($curl);
        $products = json_decode($json, true);
        $products = $products[0]['products'];

        $req = [];
        foreach ($products as $prod) {
            $req[] = 'id[]=' . $prod['id'];
        }
        $req = implode('&', $req);
        $curl = curl_init();
        curl_setopt($curl, CURLOPT_URL, 'https://api.hiconix.ru/api/v2/products?' . $req);
        curl_setopt($curl, CURLOPT_HTTPHEADER, array($this->header));
        curl_setopt($curl, CURLOPT_FOLLOWLOCATION, true);
        curl_setopt($curl, CURLOPT_MAXREDIRS, 10);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_HEADER, false);
        $json = curl_exec($curl);
        $prod_res = json_decode($json, true);
        return $prod_res;
    }



    private function setTopCat($cid = 0, $ims = '', $img = '')
    {
        if ($ims == $this->brand['logo']) {
            $image = $img;
        } else {
            $image = $this->getImage($this->brand['logo'], 'category');
        }
        if ($cid) {
            $this->db->query("
				UPDATE `" . DB_PREFIX . "category`
				SET `image` = '" . $this->db->escape($image) . "',
					`image_src` = '" . $this->db->escape($this->brand['logo']) . "'
				WHERE `category_id` = " . $cid . "
			");
            if (!empty($this->config['update_cats'])) {
                $this->db->query("
					UPDATE `" . DB_PREFIX . "category_description`
					SET `name` = '" . $this->db->escape($this->brand['name']) . "',
						`description` = '" . $this->db->escape($this->brand['description']) . "',
						`meta_title` = '" . $this->db->escape($this->brand['name']) . "'
					WHERE `category_id` = " . $cid . " AND `language_id` = " . $this->langid . "
				");
            }
            if (!empty($this->config['update_seo'])) {
                $this->db->query("
					DELETE FROM `" . DB_PREFIX . "url_alias`
					WHERE `query` = 'category_id=" . $cid . "'
				");
                $test = $this->db->query("
					SELECT `query` FROM `" . DB_PREFIX . "url_alias`
					WHERE `keyword` = '" . $this->db->escape($this->brand['slug']) . "'
				");
                if (isset($test->row['query'])) {
                    $this->brand['slug'] .= '_c' . $cid;
                }
                $this->db->query("
					INSERT INTO `" . DB_PREFIX . "url_alias`
					SET `query` = 'category_id=" . $cid . "',
						`keyword` = '" . $this->db->escape($this->brand['slug']) . "'
				");
            }
            return $cid;
        }

        $this->db->query("
			INSERT INTO `" . DB_PREFIX . "category`
			SET `hix_cid` = " . (int)$this->brand['id'] . ",
				`image` = '" . $this->db->escape($image) . "',
				`image_src` = '" . $this->db->escape($this->brand['logo']) . "',
				`column` = 4,
				`date_added` = NOW(),
				`date_modified` = NOW(),
				`status` = 1,
				`parent_id` = 0,
				`top` = 1,
				`sort_order` = " . (int)$this->brand['position'] . "
		");
        $cid = (int)$this->db->getLastId();
        $this->db->query("
			INSERT INTO `" . DB_PREFIX . "category_description`
			SET `name` = '" . $this->db->escape($this->brand['name']) . "',
				`description` = '" . $this->db->escape($this->brand['description']) . "',
				`meta_title` = '" . $this->db->escape($this->brand['name']) . "',
				`category_id` = " . $cid . ",
				`language_id` = " . $this->langid . "
		");
        $this->db->query("
			INSERT INTO `" . DB_PREFIX . "category_to_store`
			SET `category_id` = " . $cid . ", `store_id` = " . (int)$this->config['store'] . "
		");
        $this->db->query("
			DELETE FROM `" . DB_PREFIX . "url_alias`
			WHERE `query` = 'category_id=" . $cid . "'
		");
        $test = $this->db->query("
			SELECT `query` FROM `" . DB_PREFIX . "url_alias`
			WHERE `keyword` = '" . $this->db->escape($this->brand['slug']) . "'
		");
        if (isset($test->row['query'])) {
            $this->brand['slug'] .= '_c' . $cid;
        }
        $this->db->query("
			INSERT INTO `" . DB_PREFIX . "url_alias`
			SET `query` = 'category_id=" . $cid . "',
				`keyword` = '" . $this->db->escape($this->brand['slug']) . "'
		");
        return $cid;
    }

    private
    function getTopCat()
    {
        if ((int)$this->config['brands'][$this->brand_id]['category']) {
            return (int)$this->config['brands'][$this->brand_id]['category'];
        }
        $test = $this->db->query("
			SELECT `category_id` AS catid, `image_src`, `image`
			FROM `" . DB_PREFIX . "category`
			WHERE `hix_cid` = " . (int)$this->brand['id']
        );
        if (isset($test->row['catid'])) {
            if (!empty($this->config['update_cats']) || !empty($this->config['update_seo'])) {
                $this->setTopCat((int)$test->row['catid'], $test->row['image_src'], $test->row['image']);
            }
            $cid = (int)$test->row['catid'];
            $this->db->query("
				DELETE FROM `" . DB_PREFIX . "category_to_store`
				WHERE `category_id` = " . $cid . "
					AND `store_id` = " . (int)$this->config['store'] . "
			");
            $this->db->query("
				INSERT INTO `" . DB_PREFIX . "category_to_store`
				SET `category_id` = " . $cid . ",
					`store_id` = " . (int)$this->config['store'] . "
			");
            return $cid;
        }
        return $this->setTopCat();
    }

    private function setSubCat($cid = 0, $ims = '', $img = '')
    {
        if ($ims == $this->category['image']) {
            $image = $img;
        } else {
            $image = $this->getImage($this->category['image'], 'category');
        }
        if ($cid) {
            $this->db->query("
				UPDATE `" . DB_PREFIX . "category`
				SET `image` = '" . $this->db->escape($image) . "',
					`image_src` = '" . $this->db->escape($this->category['image']) . "'
				WHERE `category_id` = " . $cid . "
			");
            if (!empty($this->config['update_cats'])) {
                $this->db->query("
					UPDATE `" . DB_PREFIX . "category_description`
					SET `name` = '" . $this->db->escape($this->category['name']) . "',
						`description` = '" . $this->db->escape($this->category['description']) . "',
						`meta_title` = '" . $this->db->escape($this->category['fullname']) . "'
					WHERE `category_id` = " . $cid . " AND `language_id` = " . $this->langid . "
				");
            }
            if (!empty($this->config['update_seo'])) {
                $this->db->query("
					DELETE FROM `" . DB_PREFIX . "url_alias`
					WHERE `query` = 'category_id=" . $cid . "'
				");
                $test = $this->db->query("
					SELECT `query` FROM `" . DB_PREFIX . "url_alias`
					WHERE `keyword` = '" . $this->db->escape($this->category['slug']) . "'
				");
                if (isset($test->row['query'])) {
                    $this->category['slug'] .= '_c' . $cid;
                }
                $this->db->query("
					INSERT INTO `" . DB_PREFIX . "url_alias`
					SET `query` = 'category_id=" . $cid . "',
						`keyword` = '" . $this->db->escape($this->category['slug']) . "'
				");
            }
            return $cid;
        }

        $this->db->query("
			INSERT INTO `" . DB_PREFIX . "category`
			SET `hix_cid` = " . (int)$this->category['id'] . ",
				`image` = '" . $this->db->escape($image) . "',
				`image_src` = '" . $this->db->escape($this->category['image']) . "',
				`date_added` = NOW(),
				`date_modified` = NOW(),
				`status` = 1,
				`parent_id` = " . $this->topcat . ",
				`top` = 1,
				`sort_order` = " . (int)$this->category['position'] . "
		");
        $cid = (int)$this->db->getLastId();
        $this->db->query("
			INSERT INTO `" . DB_PREFIX . "category_description`
			SET `name` = '" . $this->db->escape($this->category['name']) . "',
				`description` = '" . $this->db->escape($this->category['description']) . "',
				`meta_title` = '" . $this->db->escape($this->category['fullname']) . "',
				`category_id` = " . $cid . ",
				`language_id` = " . $this->langid . "
		");
        $this->db->query("
			INSERT INTO `" . DB_PREFIX . "category_to_store`
			SET `category_id` = " . $cid . ", `store_id` = " . (int)$this->config['store'] . "
		");
        $this->db->query("
			DELETE FROM `" . DB_PREFIX . "url_alias`
			WHERE `query` = 'category_id=" . $cid . "'
		");
        $test = $this->db->query("
			SELECT `query` FROM `" . DB_PREFIX . "url_alias`
			WHERE `keyword` = '" . $this->db->escape($this->category['slug']) . "'
		");
        if (isset($test->row['query'])) {
            $this->category['slug'] .= '_c' . $cid;
        }
        $this->db->query("
			INSERT INTO `" . DB_PREFIX . "url_alias`
			SET `query` = 'category_id=" . $cid . "',
				`keyword` = '" . $this->db->escape($this->category['slug']) . "'
		");
        return $cid;
    }

    private function getSubCat()
    {
        if (empty($this->config['brands'][$this->brand_id]['loadcats'])) {
            return $this->topcat;
        }
        $test = $this->db->query("
			SELECT `category_id` AS catid, `image_src`, `image`
			FROM `" . DB_PREFIX . "category`
			WHERE `hix_cid` = " . (int)$this->category['id']
        );
        if (isset($test->row['catid'])) {
            if (!empty($this->config['update_cats']) || !empty($this->config['update_seo'])) {
                $this->setSubCat((int)$test->row['catid'], $test->row['image_src'], $test->row['image']);
            }
            $cid = (int)$test->row['catid'];
            $this->db->query("
				DELETE FROM `" . DB_PREFIX . "category_to_store`
				WHERE `category_id` = " . $cid . "
					AND `store_id` = " . (int)$this->config['store'] . "
			");
            $this->db->query("
				INSERT INTO `" . DB_PREFIX . "category_to_store`
				SET `category_id` = " . $cid . ",
					`store_id` = " . (int)$this->config['store'] . "
			");
            return $cid;
        }
        return $this->setSubCat();
    }

    private function setProd($pid = 0, $ims = '', $img = '')
    {
        $found = false;
        $images = $this->product['images'];
        usort($images, function ($a, $b) {
            $inta = (int)$a['position'];
            $intb = (int)$b['position'];
            if ($inta < $intb) return -1;
            if ($inta > $intb) return 1;
            return 0;
        });
        if ($ims) {
            foreach ($images as $idx => $i) {
                if ($ims == $i['url']) {
                    $found = true;
                    unset($images[$idx]);
                }
            }
        }
        if ($found) {
            $image = $img;
        } else {
            $tmp = array_shift($images);
            $ims = $tmp['url'];
            $image = $this->getImage($ims, 'product');
        }
        $price = 0;
        foreach ($this->product['parameters'] as $idx => $p) {
            if ($p['param'] == 'price') {
                $price = $p['value'];
                unset($this->product['parameters'][$idx]);
                break;
            }
        }
        if ($pid) {
            $this->db->query("
                UPDATE " . DB_PREFIX . "product
                SET `image` = '" . $this->db->escape($image) . "',
                    `image_src` = '" . $this->db->escape($ims) . "'
                WHERE `product_id` = " . $pid . "
            ");
            if ($images) {
                $this->setImages($pid, $images, true);
            }
            if (!empty($this->config['update_price'])) {
                $this->db->query("
	                UPDATE " . DB_PREFIX . "product
	                SET `price` = '" . (float)$price . "'
	                WHERE `product_id` = " . $pid . "
	            ");
            }
            if (!empty($this->config['update_desc'])) {
                $this->db->query("
					UPDATE `" . DB_PREFIX . "product_description`
					SET `name` = '" . $this->db->escape($this->product['name']) . "',
						`description` = '" . $this->db->escape($this->product['description']) . "',
						`meta_title` = '" . $this->db->escape($this->category['name']) . "'
					WHERE `product_id` = " . $pid . " AND `language_id` = " . $this->langid . "
				");
            }
            if (!empty($this->config['update_seo'])) {
                $this->db->query("
					DELETE FROM `" . DB_PREFIX . "url_alias`
					WHERE `query` = 'product_id=" . $pid . "'
				");
                $test = $this->db->query("
					SELECT `query` FROM `" . DB_PREFIX . "url_alias`
					WHERE `keyword` = '" . $this->db->escape($this->product['slug']) . "'
				");
                if (isset($test->row['query'])) {
                    $this->product['slug'] .= '_p' . $pid;
                }
                $this->db->query("
					INSERT INTO `" . DB_PREFIX . "url_alias`
					SET `query` = 'product_id=" . $pid . "',
						`keyword` = '" . $this->db->escape($this->product['slug']) . "'
				");
            }
            return $pid;
        }
        $this->db->query("
            INSERT INTO " . DB_PREFIX . "product
            SET `hix_pid` = " . (int)$this->product['id'] . ",
            	`manufacturer_id` = " . $this->manid . ",
                `image` = '" . $this->db->escape($image) . "',
                `image_src` = '" . $this->db->escape($ims) . "',
                `model` = '" . $this->db->escape($this->product['code']) . "',
                `sku` = '" . $this->db->escape($this->product['code']) . "',
                `price` = '" . (float)$price . "',
                `quantity` = 1000,
                `shipping` = 1,
                `date_available` = DATE(NOW()),
                `date_added` = NOW(),
                `date_modified` = NOW(),
                `status` = 1,
                `subtract` = 0,
                `minimum` = 1
        ");
        $pid = (int)$this->db->getLastId();
        if ($images) {
            $this->setImages($pid, $images);
        }
        $this->db->query("
            INSERT INTO " . DB_PREFIX . "product_description
            SET `product_id` = " . $pid . ",
            	`language_id` = " . $this->langid . ",
                `name` = '" . $this->db->escape($this->product['name']) . "',
                `description` = '" . $this->db->escape($this->product['description']) . "',
                `meta_title` = '" . $this->db->escape($this->product['name']) . "'
        ");
        $this->db->query("
            INSERT INTO " . DB_PREFIX . "product_to_category
            SET `product_id` = " . $pid . ",
            	`category_id` = " . $this->subcat . ",
                `main_category` = 1
        ");
        $this->db->query("
            INSERT INTO " . DB_PREFIX . "product_to_store
            SET `product_id` = " . $pid . ",
            	`store_id` = " . $this->config['store'] . "
        ");
        $this->db->query("
			DELETE FROM `" . DB_PREFIX . "url_alias`
			WHERE `query` = 'product_id=" . $pid . "'
		");
        $test = $this->db->query("
			SELECT `query` FROM `" . DB_PREFIX . "url_alias`
			WHERE `keyword` = '" . $this->db->escape($this->product['slug']) . "'
		");
        if (isset($test->row['query'])) {
            $this->product['slug'] .= '_p' . $pid;
        }
        $this->db->query("
			INSERT INTO `" . DB_PREFIX . "url_alias`
			SET `query` = 'product_id=" . $pid . "',
				`keyword` = '" . $this->db->escape($this->product['slug']) . "'
		");

        return $pid;
    }

    private function getProdId()
    {
        $test = $this->db->query("
			SELECT `product_id` AS prodid, `image_src`, `image`
			FROM `" . DB_PREFIX . "product`
			WHERE `hix_pid` = " . (int)$this->product['id']
        );
        if (isset($test->row['prodid'])) {
            if (!empty($this->config['update_price']) || !empty($this->config['update_seo']) || !empty($this->config['update_desc'])) {
                $this->setProd((int)$test->row['prodid'], $test->row['image_src'], $test->row['image']);
            }
            $pid = (int)$test->row['prodid'];
            $this->db->query("
				DELETE FROM `" . DB_PREFIX . "product_to_store`
				WHERE `product_id` = " . $pid . "
					AND `store_id` = " . (int)$this->config['store'] . "
			");
            $this->db->query("
				INSERT INTO `" . DB_PREFIX . "product_to_store`
				SET `product_id` = " . $pid . ",
					`store_id` = " . (int)$this->config['store'] . "
			");
            return $pid;
        }
        return $this->setProd();
    }

    private function getAttrs()
    {
        $retval = array();
        $pattr = array();
        foreach ($this->product['parameters'] as $pa) {
            if(isset($pa['param'])) {
                $code = $pa['param'];
                if ($code == 'price' || $code == 'code') continue;
                unset($pa['param']);
                $pattr[$code] = $pa;
            }
        }
        foreach ($this->config['attributes'] as $code => $data) {
            if ($code == 'price' || $code == 'code') continue;
            $data['value'] = false;
            switch ($code) {
                case 'country':
                    $v = '';
                    if(isset($pattr['country']['indoor_value'])) {
                        $v = 'Внутренний блок: ' . $pattr['country']['indoor_value'] . '<br>' .
                            'Наружный блок: ' . $pattr['country']['outdoor_value'];
                    } elseif(isset($pattr['country']['value'])) {
                        $v = $pattr['country']['value'];
                    }
                    $data['value'] = $v;
                    break;
                case 'coolpower':
                case 'heatpower':
                case 'heatnetpower':
                case 'coolnetpower':
                case 'gas_tube_diam':
                case 'liquid_tube_diam':
                    $v = '';
                    if(isset($pattr[$code]) && isset($pattr[$code]['value'])) {
                        $v = round((float)$pattr[$code]['value'], 2);
                        if(!empty($pattr[$code]['units'])) {
                            $v .= ' ' . $pattr[$code]['units'];
                        }
                    }
                    $data['value'] = $v;
                    break;
                case 'warranty':
                case 'inverter':
                    if(empty($pattr[$code]['value'])) {
                        $data['value'] = 'Нет';
                    } else {
                        $data['value'] = 'Да';
                    }
                    break;
                default: {
                    if(isset($pattr[$code]) && isset($pattr[$code]['value'])) {
                        $data['value'] = $pattr[$code]['value'];
                        if(!empty($pattr[$code]['units'])) {
                            $data['value'] .= ' ' . $pattr[$code]['units'];
                        }
                        if(!empty($pattr[$code]['description'])) {
                            $data['name'] = $pattr[$code]['description'];
                        }
                    } else {
                        $data['value'] = false;
                    }
                }
            }
            if ($data['value'] !== false) {
                $retval[$code] = $data;
            }
        }
        return $retval;
    }

    private function getAttrGroupId()
    {
        $tmp = $this->db->query("
			SELECT `attribute_group_id` AS agid
			FROM `" . DB_PREFIX . "attribute_group_description`
			WHERE `name` = '***'
		");
        if (isset($tmp->row['agid'])) {
            return (int)$tmp->row['agid'];
        }
        $this->db->query("
			INSERT INTO `" . DB_PREFIX . "attribute_group`
			SET `sort_order` = 0
		");
        $agid = (int)$this->db->getLastId();
        $this->db->query("
			INSERT INTO `" . DB_PREFIX . "attribute_group_description`
			SET `name` = '***',
			`attribute_group_id` = " . $agid . ",
			`language_id` = " . $this->langid . "
		");
        return $agid;
    }

    private function getAttrMap()
    {
        $retval = array();
        foreach ($this->attributes as $code => $val) {
            if ($val['map']) {
                $retval[$code] = $val['map'];
            } else {
                $test = $this->db->query("
					SELECT a.`attribute_id` AS atrid
					FROM `" . DB_PREFIX . "attribute` a
					LEFT JOIN `" . DB_PREFIX . "attribute_description` ad ON(a.`attribute_id` = ad.`attribute_id`)
					WHERE ad.`language_id` = " . $this->langid . "
						AND ad.`name` = '" . $this->db->escape($val['name']) . "'
						AND a.`attribute_group_id` = " . $this->attrgrid . "
				");
                if (isset($test->row['atrid'])) {
                    $retval[$code] = (int)$test->row['atrid'];
                    continue;
                }
                $this->db->query("
					INSERT INTO `" . DB_PREFIX . "attribute`
					SET `sort_order` = 0,
					`attribute_group_id` = " . $this->attrgrid . "
				");
                $atrid = (int)$this->db->getLastId();
                $this->db->query("
					INSERT INTO `" . DB_PREFIX . "attribute_description`
					SET `name` = '" . $this->db->escape($val['name']) . "',
						`language_id` = " . $this->langid . ",
						`attribute_id` = " . $atrid . "
				");
                $retval[$code] = $atrid;
            }
        }
        return $retval;
    }

    private function setAttrs()
    {
        foreach ($this->attributes as $code => $attr) {
            $test = $this->db->query("
				SELECT `attribute_id` AS atrid
				FROM `" . DB_PREFIX . "product_attribute`
				WHERE `language_id` = " . $this->langid . "
					AND `product_id` = " . $this->prod_id . "
					AND `attribute_id` = " . $this->attrmap[$code] . "
			");
            if (isset($test->row['atrid'])) {
                if ($attr['update']) {
                    if ($attr['value']) {
                        $this->db->query("
							UPDATE `" . DB_PREFIX . "product_attribute`
							SET `text` = '" . $this->db->escape($attr['value']) . "'
							WHERE `language_id` = " . $this->langid . "
								AND `product_id` = " . $this->prod_id . "
								AND `attribute_id` = " . $this->attrmap[$code] . "
						");
                    } else {
                        $this->db->query("
							DELETE FROM `" . DB_PREFIX . "product_attribute`
							WHERE `language_id` = " . $this->langid . "
								AND `product_id` = " . $this->prod_id . "
								AND `attribute_id` = " . $this->attrmap[$code] . "
						");
                    }
                }
            } else {
                if ($attr['value']) {
                    $this->db->query("
						INSERT INTO `" . DB_PREFIX . "product_attribute`
						SET `text` = '" . $this->db->escape($attr['value']) . "',
							`language_id` = " . $this->langid . ",
							`product_id` = " . $this->prod_id . ",
							`attribute_id` = " . $this->attrmap[$code] . "
					");
                }
            }
        }
    }





    public function debug()
    {
    }


    public function doLoad()
    {
        $this->getData();
        foreach ($this->data as $idx => &$brand) {
            if (empty($this->config['brands'][$idx]['load'])) {
                continue;
            }
            $this->brand = &$brand;
            $this->brand_id = $idx;
            $this->topcat = $this->getTopCat();
            $this->manid = $this->setManufacturer();
            foreach ($brand['categories'] as &$cat) {
                if (in_array($cat['id'], array(13, 2, 1))) {
                    $cat['id'] *= 100000;
                }
                $this->category = &$cat;
                $this->subcat = $this->getSubCat();
                foreach ($cat['products'] as $prod) {
                    $this->product = &$prod;
                    $this->prod_id = $this->getProdId();
                    $this->attributes = $this->getAttrs();
                    $this->attrgrid = $this->getAttrGroupId();
                    if (!$this->attrmap) {
                        $this->attrmap = $this->getAttrMap();
                    }
                    $this->setAttrs();
                    $percent = 100.0 * (float)(++$this->loaded_prods) / (float)$this->total_prods;
                    $this->log('PRODS', $percent, 'Бренд: ' . $brand['name'] . ', категория: ' . $cat['name'] . ', товар ' . $this->loaded_prods . ' из ' . $this->total_prods);
                }
            }
        }
        $this->repairCategories();
        $this->log('COMPLETE', false, 'Каталог Hiconix загружен и обновлен');
    }

    private function getData()
    {
        $this->getPairs();
        $this->getPairsProd();
        $total = count($this->req);
        $bcnt = 0;
        $percent = 0;
        foreach ($this->req as $brand) {
            $bdt = $this->curlHandler($brand['path']);
            if (!is_array($bdt)) {
                file_put_contents($this->logfile, json_encode(array('code' => 'ERROR', 'msg' => 'Неверный токен доступа')));
                exit(0);
            }
            $cnt = 0;
            if ($bdt && is_array($bdt)) {
                if (isset($bdt['categories'])) {
                    foreach ($bdt['categories'] as &$cat) {
                        $cat['products'] = $this->getComplects($cat['id']);
                        if (isset($cat['products'])) {
                            $cnt += count($cat['products']);
                        }
                    }
                    if ($cnt) {
                        $this->data[$brand['idx']] = $bdt;
                    }
                }
            }
            $this->total_prods += $cnt;
            $percent = round(100.0 * (float)(++$bcnt) / (float)$total);
            $this->log('LOAD', $percent, 'Бренд ' . $brand['name'] . ' загружен. Пожалуйста, подождите. Загрузка бренда занимает до 2-х минут.');
        }
    }

    private function curlHandler($url){
        $curl = curl_init();
        curl_setopt($curl, CURLOPT_URL, $url);
        curl_setopt($curl, CURLOPT_HTTPHEADER, array($this->header));
        curl_setopt($curl, CURLOPT_FOLLOWLOCATION, true);
        curl_setopt($curl, CURLOPT_MAXREDIRS, 10);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_HEADER, false);
        $json = curl_exec($curl);
        curl_close($curl);
        return json_decode($json, true);
    }

    public function getPairs(){
        $this->pairs = $this->curlHandler('https://api.hiconix.ru/api/v2/pairs');
    }

    private function getPairsProd(){
        $count = 0;
        $pairs_ids = array();
        foreach ($this->pairs as $item){
            $count += count($item['outdoors']) * count($item['indoors']);
            $pairs_ids = array_merge($pairs_ids, $item['outdoors']);
            $pairs_ids = array_merge($pairs_ids, $item['indoors']);
        }
        $pairs_ids = array_unique($pairs_ids);
        $ids = array();
        foreach ($pairs_ids as $id){
            $ids[] = $id;
            if(count($ids) == 15){
                $prods = $this->getProductsByIdApi($ids);
                foreach ($prods as $item){
                    $this->pairs_prod[$item['id']] = $item;
                }
                $ids = array();
            }
        }
        if(count($ids) > 0){
            $prods = $this->getProductsByIdApi($ids);
            foreach ($prods as $item){
                $this->pairs_prod[$item['id']] = $item;
            }
        }
    }

    private function getProductsByIdApi($arr){
        foreach($arr as $item) {
            $req[] = 'id[]=' . $item;
        }
        $req = implode('&', $req);
        return $this->curlHandler('https://api.hiconix.ru/api/v2/products?' . $req);
    }

    private function getComplects($id_cat){
        $products = $this->curlHandler('https://api.hiconix.ru/api/v2/categories?id[]=' . $id_cat);
        $products = $products[0]['products'];
        $complect_prods = array();
        foreach ($products as $prod){
            $pairs = $this->curlHandler('https://api.hiconix.ru/api/v2/pairs?id=' . $prod['id']);
            if($pairs) {
                foreach ($pairs as $pair) {
                    $complect_prods = array_merge($complect_prods, $this->pairHandler($pair));
                }
            }
        }
        return $complect_prods;
    }

    // обработка комплекта
    private function pairHandler($pair){
        $complect_prods = array();
        $prefixname = $this->config['prefixname'];
        foreach ($pair['outdoors'] as $outdoors_id) {
            foreach ($pair['indoors'] as $indoors_id) {
                if(!in_array(strval($indoors_id) . strval($outdoors_id), $this->addedPairs)){
                    $this->addedPairs[] = strval($indoors_id) . strval($outdoors_id);
                    $outdoors = $this->pairs_prod[$outdoors_id];
                    $indoors = $this->pairs_prod[$indoors_id];
                    $prod = $indoors;

                    $prod['name'] = $indoors['name'] . ' / ' . $outdoors['name'];
                    if(!empty(trim($prefixname))){
                        if(strpos($prefixname, '{name}') !== false) {
                            $prod['name'] = str_replace('{name}', $prod['name'], $prefixname);
                        } else {
                            $prod['name'] = $prefixname . $prod['name'];
                        }
                    }

                    $prod['slug'] = $indoors['slug'] . '-' . $outdoors['slug'];
                    $prod['images'] = array_merge($indoors['images'], $outdoors['images']);
                    $i = 1;
                    foreach($prod['images'] as &$img){
                        $img['position'] = $i++;
                    }

                    foreach ($prod['parameters'] as &$param){
                        if($param['param'] == 'price') {
                            $param['value'] = strval($this->getRecomPrice($indoors) + $this->getRecomPrice($outdoors));
                        }
                        if($param['param'] == 'noise') {
                            $param['param'] = 'noise_indoor';
                            $param['description'] = 'Уровень шума внутреннего блока';
                        }
                        if($param['param'] == 'weight') {
                            $param['param'] = 'weight_indoor';
                            $param['description'] = 'Вес внутреннего блока';
                        }
                    }

                    foreach ($outdoors['parameters'] as &$param){
                        if($param['param'] == 'noise') {
                            $param['param'] = 'noise_outdoor';
                            $param['description'] = 'Уровень шума наружного блока';
                            $prod['parameters'][] = $param;
                        }
                        if($param['param'] == 'weight') {
                            $param['param'] = 'weight_outdoor';
                            $param['description'] = 'Вес наружного блока';
                            $prod['parameters'][] = $param;
                        }
                    }

                    $tmp = $this->getWidthHeightLength($indoors);
                    $prod['parameters'][] = [
                        'description' => 'Размеры внутреннего блока (ШхВхД)',
                        'param' => 'widthheightlength_indoor',
                        'value' => $tmp['values'],
                        'units' => $tmp['units'],
                    ];

                    $tmp = $this->getWidthHeightLength($outdoors);
                    $prod['parameters'][] = [
                        'description' => 'Размеры наружного блока (ШхВхД)',
                        'param' => 'widthheightlength_outdoor',
                        'value' => $tmp['values'],
                        'units' => $tmp['units'],
                    ];

                    foreach ($prod['parameters'] as &$param){
                        if($param['param'] == 'noise' || $param['param'] == 'length' || $param['param'] == 'width'
                            || $param['param'] == 'height'){
                            $param = array();
                        }
                    }
                    $complect_prods[] = $prod;
                }
            }
        }
        return $complect_prods;
    }

    private function getRecomPrice($item){
        foreach ($item['parameters'] as $param){
            if($param['param'] == 'price') {
                return intval($param['value']);
            }
        }
        return 0;
    }

    private function getWidthHeightLength($item){
        $values = [];
        $units = [];
        foreach ($item['parameters'] as $param){
            if($param['param'] == 'width') {
                $values[0] = $param['value'];
                $units[0] = $param['units'];
            }
            if($param['param'] == 'height') {
                $values[1] = $param['value'];
                $units[1] = $param['units'];
            }
            if($param['param'] == 'length') {
                $values[2] = $param['value'];
                $units[2] = $param['units'];
            }
        }
        return [
            'values' => implode('x', $values),
            'units' => $units[0],
        ];
    }
}